<?php
namespace Util;

use Moment\Moment;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager\Mongo\Types\Type;
use Zeedhi\Framework\Session\Session;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;


class Utilities {

    /** @var  Session */
    protected $session;

    protected $instanceManager;
    protected $entityManager;

    const PREFIX_ORACLE = 'USR_ORG_';

    protected static $exceptionWords = array(
        'today' => 'this-day',
        'tomorrow' => 'next-day',
        'yesterday' => 'last-day'
    );

    public function __construct(Session $session, \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager, \Doctrine\ORM\EntityManager $entityManager) {
        $this->session = $session;
        $this->instanceManager = $instanceManager;
        $this->entityManager = $entityManager;
    }


    protected static function setArrayDiff($valueBefore, $cdoperador, $nmoperador, $nrorg, $dsemailoper, $now,
                                           $keyTree, $value, $collectionId, $valueName, $collectionName, $arrayDiff = array())
    {
        $arrayDiff[] = array(
            'FIELD' => $valueName,
            'TREE_VALUE' => $keyTree,
            'VALUE_BEFORE' => $valueBefore,
            'VALUE_AFTER' => $value,
            'CDOPERADOR' => $cdoperador,
            'NMOPERADOR' => $nmoperador,
            'NRORG' => $nrorg,
            'DSEMAILOPER' => $dsemailoper,
            'DATA_UPDATE' => $now,
            'collectionId' => $collectionId,
            'collectionName' => $collectionName,
        );
        return $arrayDiff;
    }

    public static function getStringOrderBy(array $orderBy) {
        $stringOrderBy = "";
        if (count($orderBy) > 0) {
            $stringOrderBy = " ORDER BY ";
            $addVirgula = false;

            foreach ($orderBy as $key => $value) {
                $stringOrderBy .= ($addVirgula ? ", " : null) . "$key $value";
                $addVirgula = true;
            }
        }

        return $stringOrderBy;
    }

    public static function fixParentsForTreeGrid(array $values,$indexPrimary = 'ID',$indexParent = 'PARENT')
    {
        $aux = array();
        foreach ($values as $item) {
            array_push($aux, $item[$indexPrimary]);
        }
        foreach ($values as &$item) {
            if (!in_array($item[$indexParent], $aux)) {
                $item[$indexParent] = null;
            }
        }
        return $values;
    }

    /**
     * @param $nrorg
     * @return array
     */
    public static function findAndConutZmartOrg($nrorg): array
    {
        $filterCriteria = new FilterCriteria("ZmartOrg", array(
                array(
                    "columnName" => "NRORG",
                    "value" => $nrorg,
                    "operator" => "=")
            )
        );
        $dataset = self::getMongoManager()->findBy($filterCriteria);
        $count = self::getMongoManager()->mongo->getCollection("ZmartOrg")->count();
        $rows = $dataset->getRows();
        return array($count, $rows);
    }

    /**
     * @param $params
     * @param ZmartFilterCriteria $filterCriteria
     */
    private static function addOrder($params, ZmartFilterCriteria $filterCriteria)
    {
        if (!empty($params['order'])) {
            if(is_array($params['order'])){
                foreach ($params['order'] as $order) {
                    list($field, $direction) = explode(".", $order);
                    if ($direction === "asc") {
                        $direction = FilterCriteria::ORDER_ASC;
                    } else {
                        $direction = FilterCriteria::ORDER_DESC;
                    }
                    $filterCriteria->addOrderBy($field, $direction);
                }
            }else{
                list($field, $direction) = explode(".", $params['order']);
                if ($direction === "asc") {
                    $direction = FilterCriteria::ORDER_ASC;
                } else {
                    $direction = FilterCriteria::ORDER_DESC;
                }
                $filterCriteria->addOrderBy($field, $direction);
            }

        }
    }

    public function indexByMongoId($rows) {
        $newRows = array();
        foreach ($rows as $row) {
            $newRows[$row['_id']] = $row;
        }
        return $newRows;
    }

    public static function setDataSetDefault(Request $request, Response $response, array $dataSet)
    {
        $response->addDataset(new DataSet(explode('/', $request->getRoutePath())[1], $dataSet));
    }

    public static function parseDataSourceZeedhiNext($data, $total = null){
        $limit = $_REQUEST['limit'] ?? null;
        $count = $limit ?? count($data);
        $total  = $total ?? $count;
        return ["pagination" => ["total" => $total, "page" => 1, "limit" => $count],"data" =>$data];

    }

    public static function loopArray($array){
        $arrayEnd = [];
        foreach ($array as $item) {
            if(is_array($item)){
                $arrayEnd = array_merge($arrayEnd, self::loopArray($item));
            }else{
                $arrayEnd[] = $item;
            }
        }
        return $arrayEnd;
    }

    public static function fixBugArrayFilterValue($valueNext){
        return self::loopArray($valueNext);
    }

    public static function parseParamNextToZmartFilterCriteria($dataSourceName,$params){
        $filter = self::parseFilterNext($params);
        $filterCriteria = new ZmartFilterCriteria($dataSourceName, $filter);
        if (!empty($params['page']) && !empty($params['limit'])) {
            $filterCriteria->setPage(intval($params['page']));
            $filterCriteria->setPageSize(intval($params['limit']));
        }
        self::addOrder($params, $filterCriteria);

        return $filterCriteria;
    }

    public static function formatZeedhiNextParams($oldParams) {
        $paramsString = explode('&', $oldParams);
        $params = array();
        if ($paramsString) {
            foreach ($paramsString as $key => $value) {
                $parts = explode('=', $value);
                if (strpos($parts[0], '%5B') !== false) {
                    $newParts = explode('%5B', $parts[0]);
                    $parts[0] = $newParts[0];
                }
                if ($parts[1] == "true") $parts[1] = true;
                else if ($parts[1] == "false") $parts[1] = false;
                if (!isset($params[$parts[0]])) {
                    $params[$parts[0]] = $parts[1];
                } else {
                    if (!is_array($params[$parts[0]])) $params[$parts[0]] = array($params[$parts[0]]);
                    array_push($params[$parts[0]], $parts[1]);
                }
            }
        }
        return $params;
    }

    public static function parseFilterNext($params){
        $search = array();
        if(!empty($params['search_in']) && is_array($params['search_in'])){
            if(array_key_exists("search", $params)){
                $searchLikeAll = implode($params['search_in'],'|');
                array_push($search, array(
                    "columnName" => $searchLikeAll,
                    "value" => $params['search'],
                    "operator" => FilterCriteria::LIKE_ALL
                ));
            }
        }
        if(!empty($params['in']) && is_array($params['in'])){
            $filterColumns = $params['in'];
            if(array_key_exists("in", $params)){
                foreach($filterColumns as $key){
                    if(!empty($params[$key])){
                        if(is_array($params[$key])){
                            $search[] = array(
                                "columnName" => $key,
                                "value" => self::fixBugArrayFilterValue($params[$key]),
                                "operator" => FilterCriteria::IN
                            );
                        }else{
                            array_push($search, array(
                                "columnName" => $key,
                                "value" => $params[$key],
                                "operator" => FilterCriteria::LIKE_I
                            ));
                        }
                    }
                }
            }
        }
        return $search;
    }


    public static function indexByProperty($property, $rows) {
        $newRows = array();
        foreach ($rows as $row) {
            $newRows[$row[$property]] = $row;
        }
        return $newRows;
    }

    public static function array_push_array(&$array,$secondArray){
        foreach($secondArray as $arr){
            array_push($array,$arr);
        }
    }
    public static function validDatePtBr($date){
        $valid = is_string($date);

        if($valid && strlen($date) != 10){
            list($dd,$mm,$yyyy) = explode('/',$date);
            if (!checkdate($mm,$dd,$yyyy)) {
                $valid = false;
            }
        }

        return $valid;
    }

    public static function indexByProperties($properties, $rows) {
        $newRows = array();
        foreach ($rows as $row) {
            foreach ($properties as $property) {
                if (isset($row[$property])) {
                    $newRows[$row[$property]] = $row;
                }
            }
        }
        return $newRows;
    }

    public static function getImportDefaultDatabase(){
        return InstanceManager::getInstance()->getParameter('importDefaultDatabase');
    }

    public function checkVersion($frontVersion) {
        $backVersion = $this->instanceManager->getParameter('backVersion');

        if ($backVersion === $frontVersion) {
            $versionOk = true;
        } else {
            $versionOk = false;
        }

        return array(
            'versionOk' => $versionOk,
            'frontVersion' => $frontVersion,
            'backVersion' => $backVersion
        );
    }

    public static function filterConditionsToBoolean($conditions, $booleanFieldList){
        foreach($conditions as &$condition){
            if(!empty($condition["columnName"]) && in_array($condition["columnName"],$booleanFieldList)){
                $condition["value"] = $condition["value"] == "true" ? true : false;
            }
        }
        return $conditions;
    }

    public static function rebuildFilterCriteriaWithDifferentConditions(FilterCriteria $filterCriteria, $conditions){
        return new FilterCriteria($filterCriteria->getDataSourceName(),$conditions,$filterCriteria->getPage(),$filterCriteria->getPageSize());
    }

    public static function extractConditionInFilter(FilterCriteria $filterCriteria, $conditions) {
        $filter = array();
        $conditions = is_array($conditions) ? $conditions : array($conditions);
        foreach ($filterCriteria->getConditions() as $condition) {
            if (in_array($condition['columnName'], $conditions)) {
                $filter[] = $condition;
            }
        }
        return $filter;
    }

    public static function getValueOfFirstCondition(array $conditions) {
        if (isset($conditions[0]) && isset($conditions[0]['value'])) {
            return $conditions[0]['value'];
        }
        return null;
    }

    public static function ensureIsArray($data) {
        return is_array($data) ? $data : array($data);
    }

    public static function decodeStructureExternalId($externalId) {
        if(!is_array($externalId)) {
            $decoded = array();
            $externalId = explode(',', $externalId);
            foreach ($externalId as $key) {
                $decodedKey = explode(':', $key);
                $decoded[$decodedKey[0]] = $decodedKey[1];
            }
            $externalId = $decoded;
        }

        return $externalId;
    }

    public static function getTimezonedNow() {
        $now = new \DateTime();
        $now->setTimezone(new \DateTimeZone('America/Sao_Paulo'));
        return $now;
    }

    public static function encodeStructureExternalId($externalId) {
        if(is_array($externalId)) {
            $encodedIds = array();
            foreach ($externalId as $keyName => $fieldKeys) {
                $encodedIds[] = "{$keyName}:{$fieldKeys}";
            }
            $externalIdString = implode(",", $encodedIds);
            $externalId = $externalIdString;
        }
        return $externalId;
    }

    public static function isNumericArray(array $array) {
        $keys = array_keys($array);
        return $keys === array_keys($keys);
    }

    /**
     * Returns true if the given predicate is true for at least one element.
     * @author kid-icarus <https://gist.github.com/kid-icarus/8661319>
     */
    public static function array_some(array $arr, callable $callback) {
        foreach ($arr as $element) {
            if ($callback($element)) {
                return true;
            }
        }
        return false;
    }

    public static function isColumnGenerated($column) {
        if(isset($column['isGenerated'])) {
            return $column['isGenerated'];
        }
        else if (isset($column['isExternal'])) {
            return !$column['isExternal'];
        }
        else {
            return true;
        }
    }

    public static function transliterateString ($text) {
        $text = str_replace(">","_",$text);
        $text = str_replace("<","_",$text);
        $text = str_replace(" ","_",preg_replace("/&([a-z])[a-z]+;/i", "$1", htmlentities(trim($text))));
        $text = preg_replace('/[^a-z0-9]/i', '_', $text);
        return $text;
    }

    public static function getIndexInArrayAssociative(array $array,$index,$value){
        foreach ($array as $key => $row){
            if($value == $row[$index]){
                return  $key;
            }
        }
        return null;
    }

    /**
     * given an array of array items, map the property of each item to an array
     * @param $array
     * @param $property
     * @return array
     */
    public static function mapArrayItemsPropertyToArray($array, $property) {
        $returnArray = array();
        foreach ($array as $item) {
            if(isset($item[$property])) {
                $returnArray[] = $item[$property];
            }
        }

        return $returnArray;
    }

    /**
     * given an array of array items, remove (by reference) the given keys from each array item
     * @param $items
     * @param $keysToClear
     */
    public static function clearArrayItemsKeys(&$items, $keysToClear) {
        foreach ($items as &$item) {
            foreach ($keysToClear as $key) {
                if (isset($item[$key])) {
                    unset($item[$key]);
                }
            }
        }
    }

    public static function startsWith($haystack, $needle) {
        $length = strlen($needle);
        return (substr($haystack, 0, $length) === $needle);
    }

    public static function endsWith($haystack, $needle) {
        $length = strlen($needle);
        if ($length == 0) {
            return true;
        }

        return (substr($haystack, -$length) === $needle);
    }

    public static function ifIssetReturn($arr, $property) {
        return isset($arr[$property]) ? $arr[$property] : null;
    }

    public function getElementInArrayFromProperty($data, $propertyName, $propertyValue) {
        foreach ($data as $item) {
            if($item[$propertyName] === $propertyValue) {
                return $item;
            }
        }

        return false;
    }

    public function replaceParamsToEvaluate($params, $objectName = 'row') {
        $paramsConcated = '';
        $explodedParams = explode(" ", $params);

        foreach ($explodedParams as $param) {
            if(substr($param, 0, 1) === '$') {
                if(strpos($param, '+') || strpos($param, '/') || strpos($param, '*') || strpos($param, '-')){
                    $paramArray = $this->prepareParamForCase($param);
                    $paramName = str_replace('$', '', $paramArray[0]);
                    $param = '$'.$objectName.'["'.$paramName.'"]';
                    return $paramsConcated . $param . $paramArray[2] . $paramArray[1];
                }else {
                    $paramName = str_replace('$', '', $param);
                    $param = '$' . $objectName . '["' . $paramName . '"]';
                }
            }else{
                $paramReplace = array(
                    "=" => "==",
                    "<>" => "!=",
                );
                if(!empty($paramReplace[$param])){
                    $param = $paramReplace[$param];
                }
            }

            $paramsConcated = $paramsConcated . $param;
        }

        return $paramsConcated;
    }
    public function prepareParamForCase($param){
        $returnArray = array();
        if(strpos($param, '+')){
            $returnArray = explode('+', $param);
            array_push($returnArray, '+');
        } else   if(strpos($param, '-')){
            $returnArray = explode('-', $param);
            array_push($returnArray, '-');
        } else   if(strpos($param, '*')){
            $returnArray = explode('*', $param);
            array_push($returnArray, '*');
        } else   if(strpos($param, '/')){
            $returnArray = explode('/', $param);
            array_push($returnArray, '/');
        }

        return $returnArray;
    }

    public function usortAsc($a, $b){
        $indexa = self::getDimensionIndex($a,$this->usortIndex);
        $indexb = self::getDimensionIndex($b,$this->usortIndex);
        $result = $this->checkIfIndexNullAndCompareGreaterThan($a, $b, $indexa, $indexb);
        if(!$result){
            $result = $a[$indexa] > $b[$indexb] ? 1 : -1;
        }
        return $result;

    }
      public function usortDesc($a, $b){
          $indexa = self::getDimensionIndex($a,$this->usortIndex);
          $indexb = self::getDimensionIndex($b,$this->usortIndex);
          $result = $this->checkIfIndexNullAndCompareLessThan($a, $b, $indexa, $indexb);
          if(!$result){
              $result = $a[$indexa] < $b[$indexb] ? 1 : -1;
          }
          return $result;
    }

    public function usortAscDate($a, $b){
        $indexa = self::getDimensionIndex($a,$this->usortIndex);
        $indexb = self::getDimensionIndex($b,$this->usortIndex);
        $result = $this->checkIfIndexNullAndCompareGreaterThan($a, $b, $indexa, $indexb);
        if(!$result){
            $result = $this->dateInverse($a[$indexa]) > $this->dateInverse($b[$indexb]) ? 1 : -1;
        }
        return $result;
    }

    public function usortDescDate($a, $b){
        $indexa = self::getDimensionIndex($a,$this->usortIndex);
        $indexb = self::getDimensionIndex($b,$this->usortIndex);
        $result = $this->checkIfIndexNullAndCompareLessThan($a, $b, $indexa, $indexb);
        if(!$result){
            $result = $this->dateInverse($a[$indexa]) < $this->dateInverse($b[$indexb]) ? 1 : -1;
        }
        return $result;
    }

    public function usortAscString($a, $b){
        $indexa = self::getDimensionIndex($a,$this->usortIndex);
        $indexb = self::getDimensionIndex($b,$this->usortIndex);
        $result = $this->checkIfIndexNullAndCompareGreaterThan($a, $b, $indexa, $indexb);
        if(!$result){
            $result = strcmp($a[$indexa], $b[$indexb]);
        }
        return $result;
    }
    public function usortDescString($a, $b){
        $indexa = self::getDimensionIndex($a,$this->usortIndex);
        $indexb = self::getDimensionIndex($b,$this->usortIndex);
        $result = $this->checkIfIndexNullAndCompareLessThan($a, $b, $indexa, $indexb);
        if(!$result){
            $result = -strcmp($a[$indexa], $b[$indexb]);
        }
        return $result;
    }

    private function checkIfIndexNullAndCompareGreaterThan($valuea, $valueb, $indexa, $indexb)
    {
        $result = null;
        if(empty($valuea[$indexa]) && !empty($valueb[$indexb])){
            $result = -1;
        } else if(!empty($valuea[$indexa]) && empty($valueb[$indexb])){
            $result = 1;
        } else if(empty($valuea[$indexa]) && empty($valueb[$indexb])){
            $result = -1;
        }
        return $result;
    }

    public static function convertStringDataBrToMongoDate($dateString){
        return Type::getType('mongoDate')->convertToDatabaseValue(self::convertStringDataBrToUTC($dateString));
    }

    public static function convertStringDataTimeToMongoDate($dateString){
        return Type::getType('mongoDate')->convertToDatabaseValue($dateString);
    }

    public static function convertValueFilerLt(&$conditions){
        foreach ($conditions as &$condition) {
            if ($condition['operator'] == ">" && is_numeric($condition['value']) ) {
                $condition['value'] = intval($condition['value']);
            }
        }
    }

    public static function convertCreateFilterDateToMongoDate(&$conditions,$columnName)
    {
        foreach ($conditions as &$condition) {
            if ($condition['columnName'] === $columnName && is_array($condition['value'])) {
                $condition['value'][0] = self::convertStringDataBrToMongoDate($condition['value'][0]);
                $condition['value'][1] = self::convertStringDataBrToMongoDate($condition['value'][1]);
            }

        }

    }

    private function checkIfIndexNullAndCompareLessThan($valuea, $valueb, $indexa, $indexb)
    {
        $result = null;
        if(empty($valuea[$indexa]) && !empty($valueb[$indexb])){
            $result = 1;
        } else if(!empty($valuea[$indexa]) && empty($valueb[$indexb])){
            $result = -1;
        } else if(empty($valuea[$indexa]) && empty($valueb[$indexb])){
            $result = -1;
        }
        return $result;
    }

    public function dateInverse ($value){
        return implode("",array_reverse(explode('/',$value)));
    }
    public static function staticDateInverse ($value){
        return implode("",array_reverse(explode('/',$value)));
    }
    public static function convertStringDataBrToUTC ($value){
        return implode("-",array_reverse(explode('/',$value)));
    }

    public function isDate($dimension,$columns){
        foreach($columns as $column){
            if(isset($column['isDate']) && $column['name'] === $dimension){
                return $column['isDate'];
            }
        }
        return false;
    }

    public static function getDimensionIndex($array,$dimension){
        if(!isset($array[$dimension]) && isset($array['suffix'])){
            $dimension .= "_".$array['suffix'];
        }
        return $dimension;
    }

    public function orderUsort ($array, $dimension = 'ID', $orderBy, $columns){
        $orderBy = isset($orderBy) ? $orderBy : 'ASC';
        $this->usortIndex = $dimension;
        $orderColumn = $columns[array_search($dimension, array_column($columns,"name"))];
        $datatype = empty($orderColumn) || empty($orderColumn["datatype"]) ? "A" : $orderColumn["datatype"];
        if(!isset($array[0][$dimension]) && isset($array[0]['suffix'])){
            $dimension .= "_".$array[0]['suffix'];
        }
        if(count($array)){
            if(($datatype == "A" && $this->isDate($dimension,$columns)) || ($datatype == "D")){
                if($orderBy === "ASC" || $orderBy === "asc"){
                    usort($array,array("\Util\Utilities","usortAscDate"));
                } else if($orderBy === "desc"|| $orderBy === "DESC") {
                    usort($array,array("\Util\Utilities","usortDescDate"));
                }

            }else if(($datatype == "A" && isset($array[0][$dimension]) && is_numeric($array[0][$dimension])) || ($datatype == "N")){
               if($orderBy === "ASC" || $orderBy === "asc"){
                    usort($array,array("\Util\Utilities","usortAsc"));
               } else if($orderBy === "desc"|| $orderBy === "DESC") {
                    usort($array,array("\Util\Utilities","usortDesc"));
               }

           }else{
               if($orderBy === "ASC"|| $orderBy === "asc"){
                    usort($array,array("\Util\Utilities","usortAscString"));
               } else if($orderBy === "desc"|| $orderBy === "DESC") {
                    usort($array,array("\Util\Utilities","usortDescString"));
               }
           }
        }
        return $array;
    }
    public function orderMultiSort ($array, $dimensions = [], $orderBy = [],$columns){
        $reverseDimensions = array_reverse($dimensions);
        $reverseOrderBy = array_reverse($orderBy);
        foreach($reverseDimensions as $index => $dimension) {
            $this->orderUsort($array, $dimension, $reverseOrderBy[$index], $columns);
        }
        return $array;
    }
    /** @return \Controller\Periods */
    public static function getPeriodController(){
        return InstanceManager::getInstance()->getService('\Controller\Periods');
    }

    public static function setPeriod(array $dataSet)
    {
        $periods = self::getPeriodController()->findAllPeriods();
        foreach ($dataSet as &$data) {
            if(isset($data['period'])){
                foreach ($periods as $period) {
                    if(is_array($data['period'])){
                        foreach ($data['period'] as $key => $datum) {
                            if ($datum == $period['periodsCode'] && isset($period['interval'])) {
                                $data['period'][$key] = $period['_id'];
                            }
                        }
                    }else{
                        if ($data['period'] == $period['periodsCode'] && isset($period['interval'])) {
                            $data['period'] = $period['_id'];
                        }
                    }

                }
            }
        }
        return $dataSet;
    }


    public static function getValueInFilterDataSourceByName(array &$filters,$nameFilter,$remove = false)
    {
        foreach ($filters as $key => $filter) {
            $name = $filter['columnName'];
            $values = $filter['value'];
            if($name == $nameFilter ){
                if($remove){
                    unset($filters[$key]);
                }
                return $values;
            }
        }
        return null;
    }

    /** @return \Util\Utilities */
    public static function getEnvironment(){
        return InstanceManager::getInstance()->getService('\Util\Utilities');
    }

    /** @return \Teknisa\Libs\Service\Api */
    public static function getLibsServiceApi(){
        return InstanceManager::getInstance()->getService('apiService');
    }

    /** @return \Teknisa\Libs\Util\OAuthCustom */
    public static function getOAuthCustom()
    {
        return InstanceManager::getInstance()->getService('oAuthImpl');
    }

    /** @return \Util\Environment */
    public static function getEnvironmentZmart(){
        return InstanceManager::getInstance()->getService('environment');
    }

    /** @return \Doctrine\ORM\EntityManager */
    public static function getEntityManager(){
        return InstanceManager::getInstance()->getService('entityManager');
    }

    /** @return ConnectionCustom */
    public static function getConnectionCustom(){
        return InstanceManager::getInstance()->getService('\Util\ConnectionCustom');
    }

    private static function buildBetweenDatesCriteria($values) {
        $operator = 'BETWEEN';
        $dateFormat = 'DD/MM/YYYY HH24:MI:SS';
        if ($values[0] && $values[1]) {
            $values = "TO_DATE( '$values[0] 00:00:00' , '$dateFormat')
                       AND TO_DATE( '$values[1] 23:59:59' , '$dateFormat')";
        } else if ($values[0]) {
            $operator = '>';
            $values = "TO_DATE( '$values[0] 00:00:00' ,'$dateFormat')";
        } else if ($values[1]) {
            $operator = '<';
            $values = "TO_DATE( '$values[1] 00:00:00' ,'$dateFormat')";
        } else {
            $values = "";
        }

        return array($operator, $values);
    }

    private static function buildBetweenCriteria($values) {
        $operator = 'BETWEEN';
        if ($values[0] && $values[1]) {
            $values = "$values[0] AND $values[1]";
        } else if ($values[0]) {
            $operator = '>';
            $values = "$values[0]";
        } else if ($values[1]) {
            $operator = '<';
            $values = "$values[1]";
        } else {
            $values = "";
        }

        return array($operator, $values);

    }

    public static function createQueryFiltered($filters, $tableAlias = null)
    {
        $filterSQL = '';
        foreach ($filters as $filter) {
            $name = $filter['columnName'];
            $operator = $filter['operator'];
            $values = $filter['value'];

            if($operator == 'LIKE_I'){
                $operator = 'LIKE';
            }
            if($operator == 'NOT_IN'){
                $operator = 'NOT IN';
            }

            if (($name != '*' && !empty($values)) || $values == 0) {

                if ($operator == 'BETWEEN' || $operator == 'BETWEEN_DATES') {

                    if($operator == 'BETWEEN_DATES') {
                        list($operator, $values) = self::buildBetweenDatesCriteria($values);
                    } else if ($operator == 'BETWEEN') {
                        list($operator, $values) = self::buildBetweenCriteria($values);
                    }

                    if ($values || ($values && ($values[0] || $values[1]))) {
                        if (!empty($tableAlias)) {
                            $filterSQL .= " AND $tableAlias.$name $operator $values";
                        } else {
                            $filterSQL .= " AND $name $operator $values";
                        }
                    }
                } else {
                    if (is_array($values) && $operator != 'MAPPED_LIKE_ALL') {
                        $stringValues = "('" . implode($values, "','") . "')";
                        $values = $stringValues;
                    }
                    if($operator == 'LIKE' || $operator == 'LIKE_ALL' ) {

                        $values = "UPPER('" . $values . "')";

                        if ($operator == 'LIKE' ) {
                            if (!empty($tableAlias)) {
                                $name = "UPPER($tableAlias.$name)";
                            } else {
                                $name = "UPPER($name)";
                            }

                            $filterSQL .= " AND $name $operator $values";
                        } else if ($operator == 'LIKE_ALL') {

                            $columnList = explode('|', $name);
                            $likes = array();

                            foreach ($columnList as $column) {
                                if (!empty($tableAlias)) {
                                    $likes[] = "UPPER($tableAlias.$column) LIKE $values";
                                } else {
                                    $likes[] = "UPPER($column) LIKE $values";
                                }
                            }

                            $filterLikeAll = implode($likes, ' OR ');
                            $filterSQL .= " AND ( $filterLikeAll )";
                        }
                    } else if ($operator == 'MAPPED_LIKE_ALL') {
                        foreach($values[0] as $index => $mappedLikeAllValues) {
                            if($index == '_ALL') {
                                $values = "UPPER('" . $mappedLikeAllValues . "')";

                                $columnList = explode('|', explode('#', $name)[0]);
                                $likes = array();
                                foreach ($columnList as $column) {
                                    if (empty($tableAlias)) {
                                        $likes[] = "UPPER($column) LIKE $values";
                                    } else {
                                        $likes[] = "UPPER($tableAlias.$column) LIKE $values";
                                    }
                                }
                                $filterLikeAll = implode($likes, ' OR ');
                                $filterSQL .= " AND ( $filterLikeAll )";
                            } else {
                                $stringValues = "('" . implode($mappedLikeAllValues, "','") . "')";
                                if (!empty($tableAlias)) {
                                    $filterSQL .= " AND $tableAlias.$index IN $stringValues ";
                                } else {
                                    $filterSQL .= " AND $index IN $stringValues ";
                                }
                            }
                        }
                    } else {
                        if (!empty($tableAlias)) {
                            $filterSQL .= " AND $tableAlias.$name $operator $values";
                        } else {
                            $filterSQL .= " AND $name $operator $values";
                        }
                    }
                }
            }
        }

        return $filterSQL;
    }

    public function getCurrentOrganizationId()
    {
        return $this->session->has("ORGANIZATION_ID") ? $this->session->get("ORGANIZATION_ID") : null;
    }

    public function getCurrentNrOrg()
    {
        return $this->session->has("NRORG") ? $this->session->get("NRORG") : null;
    }

    public function setOrganizationId($orgId)
    {
        $this->session->set('ORGANIZATION_ID', $orgId);
    }

    public function setBaseConnection($baseConnection)
    {
        $this->session->set('baseConnection', $baseConnection);
    }

    public function getBaseConnection()
    {
        return $this->session->get('baseConnection');
    }

    public function setNrOrg($orgId)
    {
        $this->session->set('NRORG', $orgId);
    }

    public function getCurrentUserId() {
        return InstanceManager::getInstance()->getService('environment')->getCurrentUserId();
    }

    public static function getPaginatedQuery($query, $limit, $offset)
    {
        if ($limit > 0) {
            $max = $limit;
            $column = '*';
            if ($offset > 0) {
                $min = $offset + 1;
                $query = 'SELECT * FROM (SELECT a.' . $column . ', rownum AS doctrine_rownum FROM (' .
                    $query .
                    ') a WHERE rownum <= ' . $max . ') WHERE doctrine_rownum >= ' . $min;
            } else {
                $query = 'SELECT a.' . $column . ' FROM (' . $query . ') a WHERE ROWNUM <= ' . $max;
            }
        }
        return $query;
    }

    /**
     * @return \Service\Manager\Mongo\Manager
     */
    public static function getMongoManager(){
        return InstanceManager::getInstance()->getService('mongoManagerService');
    }

    public static function getNrorgoper(){
        return self::getEnvironmentZmart()->getNrorgoper();
    }

    public static function getOrganizationIdForFindAcl()
    {
        $environment = self::getEnvironmentZmart();
        $currentNrOrg = $environment->getNrOrgClient();
        $organizationId = $environment->getOrganizationId();
        if ($currentNrOrg && $organizationId == '20') {
            $organizationId = $currentNrOrg;
        }
        return $organizationId;
    }

    public static function getNrorgPrincipal()
    {
        $environment = self::getEnvironmentZmart();
        $currentNrOrg = $environment->getNrOrgClientPrincipal();
        $organizationId = $environment->getOrganizationId();
        if ($currentNrOrg && $organizationId == '20') {
            $organizationId = $currentNrOrg;
        }else{
            $organizationId = null;
        }
        return $organizationId;
    }

    public static function isTeknisa(){
        $environment = self::getEnvironmentZmart();
        return ($environment->getNrOrgClient() && $environment->getNrOrgClient() != $environment->getOrganizationId());
    }

    public static function validateOrganization($nrorg){
        $validate = true;
        list($count, $rows) = self::findAndConutZmartOrg($nrorg);
        if(empty($rows[0]['NRORG']) && $count){
            /** @var \Service\Organization $organizationService */
            $organizationService = InstanceManager::getInstance()->getService('\Service\Organization');
            $organizationService->setAllOrganization(true);
            list($count, $rows) = self::findAndConutZmartOrg($nrorg);
            if(empty($rows[0]['NRORG']) && $count){
                $validate = false;
            }
        }
        return $validate;

    }

    public static function getOrganizationIdForMongo()
    {
        $environment = self::getEnvironmentZmart();
        $currentNrOrg = $environment->getNrOrgClient();
        $organizationId = $environment->getOrganizationId();
        $nrorgoper = $environment->getNrorgoper();
        if($nrorgoper){
            $organizationId = $nrorgoper;
        }
        if ($currentNrOrg && $organizationId == '20') {
            $organizationId = $currentNrOrg;
        }
        return $organizationId;
    }

    public function contains($needle, $haystack)
    {
        return strpos($haystack, $needle) !== false;
    }

    /**
     * @return \Util\Utilities
     */
     public static function getUtilities(){
        return InstanceManager::getInstance()->getService('\Util\Utilities');
    }

    public static function getArrayFromAssociativeArray( array $array = array(), $key = 'ID')
    {
        $result = array();
        foreach ($array as $row) {
            $result[] = $row[$key] ;
        }
        return $result;
    }

    public static function objectToArray($object)
    {
        if (is_object($object) && (
                'MongoDB\Model\BSONDocument' == get_class($object) ||
                'MongoDB\Model\BSONArray' == get_class($object))
        ) {
            $return = iterator_to_array($object);
            foreach ($return as $key => $row) {
                if (is_object($row)) {
                    $return[$key] = self::objectToArray($row);
                }
            }
        } else {
            $return = $object;
            if (is_array($return)) {
                foreach ($return as $key => $row) {
                    if (is_object($row)) {
                        $return[$key] = self::objectToArray($row);
                    }
                }
            }
        }
        return $return;
    }

    public static function getStringForArray($array,$glue = ',',$start = '',$end = '',$default = ''){
        $return = $default;
        if(!empty($array)) {
            $return = $start . implode($glue, $array) . $end;
        }
        return $return;
    }

    public static function datasetMongoToZeedhi(Response $response, DataSet $dataSet,$dataSourceName)
    {
        if ($dataSet->getRows() == NULL) {
            $response->addDataSet($dataSet);
        } else {
            $rows = array();
            foreach ($dataSet->getRows() as $key => &$row) {
                foreach ($row as &$data) {
                    if ($data instanceof \DateTime) {
                        $data = $data->format('d/m/Y H:i:s');
                    }
                    if (is_array($data)) {
                        $data = json_encode($data);
                    }
                }
                $rows[$key] = $row;
            }
            $response->addDataSet(new DataSet($dataSourceName, $rows));
        }
    }

    public static function setLog($dataBefore, $dataAfter, $dataSourceName, $collectionId,$addTree = '',$mongoManager = null, $extraFields = [])
    {
        /** @var \Util\Environment $environment */
        $environment = InstanceManager::getInstance()->getService("environment");
        $cdoperador = $environment->getCdOperador();
        $nmoperador = $environment->getCurrentUserName();
        $nrorg = $environment->getNrOrg();
        $dsemailoper = $environment->getCurrentEmail();
        $now = Type::getType('mongoDate')->convertToDatabaseValue($environment->getCurrentTime()->format('Y-m-d H:i:s'));
        if (!empty($dataAfter[0]) && !empty($dataBefore[0])) {
            $arrayDiff = Utilities::buildLog($dataBefore[0], $dataAfter[0], $dataSourceName, $cdoperador, $nmoperador,
                $nrorg, $dsemailoper, $now, $collectionId, array(),$dataSourceName.$addTree);
        }else if (!empty($dataAfter[0]) && empty($dataBefore[0])){
            $arrayDiff = self::setArrayDiff('Create', $cdoperador, $nmoperador, $nrorg, $dsemailoper,
                $now, $dataSourceName.$addTree, $dataAfter, $collectionId, '',$dataSourceName);
        } else if (empty($dataAfter[0]) && !empty($dataBefore[0])){
            $arrayDiff = self::setArrayDiff($dataBefore[0], $cdoperador, $nmoperador, $nrorg, $dsemailoper,
                $now, $dataSourceName.$addTree, 'Delete', $collectionId, '',$dataSourceName);
        }
        if(!empty($arrayDiff)){
            if(!empty($extraFields)){
                foreach($arrayDiff as &$diff){
                    $diff = array_merge($diff, $extraFields);
                }
            }
            if(empty($mongoManager)){
                /** @var \Controller\ZmartLog $controllerZmartLog */
                $controllerZmartLog = InstanceManager::getInstance()->getService("\Controller\ZmartLog");

                $dataSet = new DataSet('zmartLog', $arrayDiff);
                $request = new Request\DataSet($dataSet, 'save', '/log', $cdoperador);
                $response = new Response();
                $controllerZmartLog->save($request,$response);
            }else{
                $mongoManager->persist(new DataSet('zmartLog', $arrayDiff));
            }
        }


    }

    public static function getEntityManagerWallet($wallet){
        return self::getConnectionCustom()->getConnectionWallet($wallet);
    }

    public static function getUseWalletDirect(){
        return InstanceManager::getInstance()->getParameter('useWalletDirect');
    }

    public static function getUrlAPiFindBigData(){
        return InstanceManager::getInstance()->getParameter('urlAPiFindBigData');
    }
    public static function getUrlAPiDocument(){
        return InstanceManager::getInstance()->getParameter('urlAPiDocument');
    }

    public static function validateWallet($url)
    {
        $wallet = str_replace('jdbc:oracle:thin:@','',$url);
        $return = false;
        $organizationId = self::getOrganizationIdForMongo();
        if ($wallet == self::PREFIX_ORACLE . $organizationId) {
            $return = true;
        }else{
            $pos = strrpos($wallet,self::PREFIX_ORACLE);
            if($pos === false){
                $return = true;
            }
        }
        return $return;

    }

    public static function getWallet(array $newConnectionParams)
    {
        $url = OpensslCrypt::decrypt($newConnectionParams['dbUrlSsl']);
        $wallet = str_replace('jdbc:oracle:thin:@', '', $url);
        return $wallet;
    }

    public static function execQueryWallet($query, $dataSourceName, $wallet)
    {
        $entityManagerWallet = Utilities::getEntityManagerWallet($wallet);
        $query = Utilities::injectIdentification($query, $dataSourceName);
        $queryResult = $entityManagerWallet->getConnection()->fetchAll($query);
        return  $queryResult;
    }


    public static function injectIdentification($query, $name)
    {
        $organizationId = self::getOrganizationIdForMongo();
        $queryIdentification = <<<SQL
/* ZmartBI - $organizationId - $name */

SQL;
        return $queryIdentification . $query;
    }

    public static function getOrganizationUseWallet($wallet){
         $return = false;
         $useWalletDirect = self::getUseWalletDirect();
         if($useWalletDirect){
             $organizationId = self::getOrganizationIdForMongo();
             if(self::PREFIX_ORACLE.$organizationId === $wallet){
                 $typeHosting = self::getOrganizationHosting();
                 $return = $typeHosting == 'S';
             }
         }
         return $return;
    }

    public static function getOrganizationHosting(){
        $organizationId = self::getOrganizationIdForMongo();
        $query = <<<SQL
            SELECT IDORGSAAS FROM ORGANIZACAO 
            WHERE NRORG = $organizationId
SQL;
        $organization = self::getEntityManager()->getConnection()->fetchAll($query);
        return empty($organization)? null : $organization[0]['IDORGSAAS'];
    }

    protected static $fieldNotLogs = array(
        'dataSource' => array('lastUpdate','__isSelected','_queryChanged', '__related', '_originDatasourcesChanged', '_queryBkp'),
        'indicator' => array('dataSourceColumns'),
        'dashboard' => array('changed'),
        'structure' => array('changed', 'generated', 'typeName')
    );

    public static function buildLog($dataBefore, $dataAfter, $collectionName, $cdoperador, $nmoperador, $nrorg, $dsemailoper,
                                    $now, $collectionId, $arrayDiff = array(), $keyTree = '', $valueName = ''){

        $indexIgnored = empty(self::$fieldNotLogs[$collectionName]) ? array() : self::$fieldNotLogs[$collectionName];
        if (is_array($dataBefore)) {
            if(is_array($dataAfter)){
                foreach($dataAfter as $key => $value) {
                    if(empty($dataBefore[$key])){
                        $dataBefore[$key] = is_array($value) ? [] : null;
                    }
                }
            }
            foreach ($dataBefore as $key => $value) {
                if (!in_array($key, $indexIgnored,true)) {
                    $valueAfter = empty($dataAfter[$key]) ? null : $dataAfter[$key];
                    $valueName = empty($dataAfter['name']) ? '' : $dataAfter['name'];
                    $valueName = empty($valueName) && !empty($dataBefore[$key]['name']) ? $dataBefore[$key]['name']: '';
                    $arrayDiff = self::buildLog($value, $valueAfter, $collectionName, $cdoperador, $nmoperador, $nrorg,
                        $dsemailoper, $now,$collectionId, $arrayDiff, $keyTree . '_' . $key,$valueName);
                }
            }
        } else if ($dataAfter != $dataBefore) {
            $arrayDiff = self::setArrayDiff($dataBefore, $cdoperador, $nmoperador, $nrorg, $dsemailoper,
                $now, $keyTree, $dataAfter, $collectionId, $valueName,$collectionName, $arrayDiff);

        }
        return $arrayDiff;
    }
}
