import { Modal, ModalService, Form, AlertService, DialogService } from '@zeedhi/common';
import { Metadata, Http } from '@zeedhi/core';

export class ChangeClienteController {
  private modal!: Modal;

  public async openModal() {
    if (this.modal !== undefined) {
        await this.modal?.destroy();
    }
    this.modal = await ModalService.createFromJson('changeClient/modal/changeClient', true);
    this.modal.show();
  }

  public async closeModal() {
    const modal = <Modal>ModalService.getModal("changeClienteModal");
    modal.hide();
    await modal.destroy();
  }

  private events: any = {
    click: () => DialogService.hide(),
  };

  private eventSave: any = {
    click: () => this.saveChangeClient(),
  };

  public confirmChangeClient() {
    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: 'Trocar Cliente',
      text: 'Deseja realmente trocar o cliente?',
      buttons: [
        { name: 'yes', component: 'ZdButton', label: 'YES', events: this.eventSave, outline: true },
        { name: 'no', component: 'ZdButton', label: 'NO', events: this.events },
      ],
    });
  }

  public async saveChangeClient() {
    const form = Metadata.getInstance<Form>("changeClientForm");
    if (!form.validate()) {
      return;
    }
    await Http.post('/zmartNrOrgCliente', form.value);
    this.closeModal();
    DialogService.hide();
    location.reload();
    AlertService.show({
            name: "edit-alert",
            text: "Estilo criado com sucesso!",
            color: "success"
        });
  }

  public async changeClientOnMounted({component} : any) {
     const form = component;
     const formValues = form.value;
     const data = await Http.get('/currentCliente');
     const dadosForm = data.data.dataset.currentCliente[0];
     formValues.CDCLIENTE = dadosForm.CDCLIENTE;
     formValues.NRINSJURCLIE = this.maskFormat(dadosForm.NRINSJURCLIE);
     formValues.NMFANTCLIE = dadosForm.NMFANTCLIE;
     formValues.NMRAZSOCCLIE = dadosForm.NMRAZSOCCLIE;
     formValues.NRORG = dadosForm.NRORG;
     formValues.NRORGPRINCIPAL = dadosForm.NRORGPRINCIPAL;
  }

    public cdClienteOnChange(params: any) {
        const form = Metadata.getInstance<Form>("changeClientForm");
        if (params.component.selectedData) {
            form.value.NRINSJURCLIE = this.maskFormat(params.component.selectedData.NRINSJURCLIE);
            form.value.NMFANTCLIE = params.component.selectedData.NMFANTCLIE;
            form.value.NMRAZSOCCLIE = params.component.selectedData.NMRAZSOCCLIE;
            form.value.NRORG = params.component.selectedData.NRORG;
            form.value.NRORGPRINCIPAL = params.component.selectedData.NRORGPRINCIPAL;
        } else {
            form.value.NRINSJURCLIE = null;
            form.value.NMFANTCLIE = null;
            form.value.NMRAZSOCCLIE = null;
            form.value.NRORG = null;
            form.value.NRORGPRINCIPAL = null;
        }
    }

  public maskFormat(value: any) {
    if (value) {
            const length = value.length;
            if (length === 11) {
              value = value.replace(/^(\d{3})(\d{3})(\d{3})(\d{2})/, "$1.$2.$3-$4");
            }
            if (length === 14) {
              value = value.replace(/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2})/, "$1.$2.$3/$4-$5");
            }
        }
    return value;
  }
}
